#include "FINDER.h"

int getoffset(functor f, int argument[])
{
  int i;
  int offset = 0;

  for (i=0; i<f->adicity; i++) {
    if (f->arg_type[i] && argument[i] >= f->arg_type[i]->siz)
      return -1;
    offset += (argument[i] * f->fats[i]);
  }
  return offset;
}



int vec_pos(functor f, int argument[])
{
  int specialcase[2];

  if ( f->comm ) {
    if ( argument[0] < argument[1] ) {
      specialcase[0] = argument[1];
      specialcase[1] = argument[0];
      return vec_pos(f,specialcase);
    }
    if (argument[0] >= f->arg_type[0]->siz)
      return (f->fcc - 1);
    return (f->fcc + ((argument[0]*(argument[0]+1))/2) + argument[1]);
  }

  return f->fcc + getoffset(f,argument);
}



functor un_vec_pos( int x, int argument[] )
{
  functor f;
  int i;
  int temp;

  for ( f = FN; f; f = f->nextfunct )
    if ( !f->piped && f->fcc <= x ) {
      if (f->comm) {
	for (argument[0] = 0;
	     argument[0] <= f->arg_type[0]->siz; 
	     argument[0]++)
	  for (argument[1] = 0; 
	       argument[1] <= argument[0]; 
	       argument[1]++)
	    if (vec_pos(f,argument) == x)
	      return f;
	skipout("Bad vector position reported",SKIP);
      }
      temp = x - f->fcc;
      for (i=f->adicity-1; i>=0; i--) {
	for (argument[i] = 0;
	     (argument[i]+1)*f->fats[i] < x;
	     argument[i]++);
	x -= argument[i];
      }
      if (!x) return f;
      skipout("Bad vector position reported",SKIP);
    }
  return 0;
}



/*
*	This simply counts the non-constant symbols in the latest
*	clause (merging duplicates) and assigns the result to its 
*	"weight" field. It also counts the constituent variables 
*	for its "varcount".
*/


void get_weight()
{
    wff_stack wf;
    int wt;
    functor f;
    boolean piped_content = false;

    wt = 0;
    for ( wf = latest_clause.testlist; wf; wf = wf->nxt ) {
	f = wf->w->sym;
	if ( f->piped ) piped_content = true;	
	if ( is_open(f) && f->category != Const )
	    wt++;
    }
    if ( !(latest_clause.weight = wt) && !piped_content )
	inerror("Clause contains no function symbol",""); 

    latest_clause.varcount = 0;
    for ( wf = latest_clause.varlist; wf; wf = wf->nxt )
	latest_clause.varcount++;
}



/*
*	We have just added a new clause to the start of the theory
*	list and set TH to point at it. So move it into the middle
*	of the list until it is in its correct position, then move
*	TH to point back at the start of the list. Note that this 
*	version of normalize assumes that the clauses are added one
*	by one, and that the pre-existing list is already normal.
*/

void normalize()
{
  while ( is_after(TH[current_theory], 
		   TH[current_theory]->nextclause) ) {
    TH[current_theory]->nextclause->lastclause = 
      TH[current_theory]->lastclause;
    
    if ( TH[current_theory]->lastclause )
      TH[current_theory]->lastclause->nextclause = 
	TH[current_theory]->nextclause;
    
    TH[current_theory]->lastclause = TH[current_theory]->nextclause;
    
    TH[current_theory]->nextclause = 
      TH[current_theory]->lastclause->nextclause;
    
    TH[current_theory]->lastclause->nextclause = TH[current_theory];
    
    if ( TH[current_theory]->nextclause )
      TH[current_theory]->nextclause->lastclause = TH[current_theory];
  }
  while ( TH[current_theory]->lastclause )
    TH[current_theory] = TH[current_theory]->lastclause;
}




/*
*	Clause c1 is after clause c2 if its weight, variable count 
*	and priority say so. This is where the order of clauses is 
*	determined. Note that the latest clause is not included.
*
*	A clause of pre-testable weight beats one of greater weight
*	irrespective of any other conditions.
*
*	If both are pre-testable the lighter wins, then the one with 
*	fewer variables, then the greater priority, then the 
*	accession order.
*
*	If neither is pre-testable the greater priority wins, then 
*	weight followed by variables (or variables followed by 
*	weight if vars_over_wt is set), then accession order.
*/

boolean is_after( clause c1, clause c2 )
{
    if ( !c1 || !c2 ) return false;

    if ( c2->weight < setting.pretest_limit ) {
	if ( c1->weight > c2->weight ) return true;
	if ( c1->weight < c2->weight ) return false;
	if ( c1->varcount > c2->varcount ) return true;
	if ( c1->varcount < c2->varcount ) return false;
	if ( c1->priority > c2->priority ) return true;
	if ( c1->priority < c2->priority ) return false;
	return ( c1 > c2 );
    }
    if ( c1->weight < setting.pretest_limit ) return false;
    
    if ( c1->priority < c2->priority ) return true;
    if ( c1->priority > c2->priority ) return false;

    if ( setting.vars_over_wt ) {
	if ( c1->varcount > c2->varcount ) return true;
	if ( c1->varcount < c2->varcount ) return false;
	if ( c1->weight > c2->weight ) return true;
	if ( c1->weight < c2->weight ) return false;
    }
    else {
	if ( c1->weight > c2->weight ) return true;
	if ( c1->weight < c2->weight ) return false;
	if ( c1->varcount > c2->varcount ) return true;
	if ( c1->varcount < c2->varcount ) return false;
    }
    return ( c1 > c2 );
}




boolean set_up_cc()
{
  set_sum();
  set_E();
  if (!set_cc())
    return false;
  tr_parameters.vlength = 0;
  if (!set_up_for(FN))
    return false;
  if (tr_parameters.vlength >= V_LENGTH) {
    fprintf(stderr,"Errrrk!!\n");
    return false;
  }
  if (!set_table(FN))
    return false;

  set_matrix(TH[current_theory]);
  if (latest_clause.prem || latest_clause.conc)
    set_matrix(&latest_clause);

#ifdef DEBUG
  { functor f;
  for ( f = FN; f; f = f->nextfunct ) if ( !f->piped ) print_set( f );
  }
#endif

  if ( !(setting.pm.verbosity & pstats) )
    tr_parameters.verb = 0;
  else if ( setting.pm.verbosity & pbstats )
    tr_parameters.verb = 1;
  else if ( setting.pm.verbosity & pfstats )
    tr_parameters.verb = 2;
  tr_parameters.Nosecs = setting.Nosecs;
  tr_parameters.topsoff = setting.topsoff;
  tr_parameters.report_breaks = setting.report_breaks;
  tr_parameters.batches = setting.batches;
  tr_parameters.extra_batches = setting.extra_batches;
  tr_parameters.maxref = setting.maxref;
  tr_parameters.maxbak = setting.max_backtracks;
  ttr_par.maxref = setting.test_maxref;
  return true;
}




/*
 *  This sets the "matrix" field of each formula to point at the table
 *  of values for the corresponding symbol.
 */

void set_matrix(clause c)
{
  if (!c) return;
  set_matrix(c->nextclause);
  set_my_matrix(c->prem);
  set_my_matrix(c->conc);
}

void set_matrices_flat(wff w); // prototype

void set_my_matrix(wff_stack ws)
{
  if (!ws) return;
  set_my_matrix(ws->nxt);
  //set_matrices_flat(ws->w);
  set_matrices(ws->w);
}
#if 0
void set_matrices_flat(wff w)
  /*------------------------------------------------------------------
   * Called by set_my_matrix().  
   * 
   * Flattened version of set_matrices().  Here a wff has its matrirx
   * set iff all its arguments have theres set.  Initialises the
   * required variables on the fly.
   * -----------------------------------------------------------------*/
{
  int arg, arg_max;
  boolean backup = false;
  wff_stack w_list;
  
  w_list->w = w; 
  w_list->next = NULL;

  while (w) { 

    if (!backup && w->sym->category == Func) {
      // set pointers to its args
      
      arg_max = w->sym->adicity - 1;
      for (arg = 0, sub = w->sb; 
	   arg < arg_max; 
	   arg++, sub++ ) {
	
      }
      w->sb[arg_max]->parent = w;
      
      w->sb[arg_max]->next_arg = NULL;
      
      // go down one level
      w = w->sb[0];
      backup = false;	
    }
    else {
      w->matrix = w->sym->table;      // set the matrix
      if (this_level->nxt) 	{
	backup = false;	
	w = this_level->next_arg;
      }
      else {
	w = w->parent;
	if (!w->parent) break;
	// need to delete the stack

	backup = true;
      }
    }
  }
} /* set_matrices_flat()---------------------------------------------------*/
#endif

void set_matrices(wff w)
{
  int i;
  
  if (!w) return;
  for (i=0; i<w->sym->adicity; i++)
    set_matrices( w->sb[i] );
  w->matrix = w->sym->table; 

} /* set_matrices() */



/*
 * The following function mallocs memory for the table of values for
 * each function. The built-in ones for each sort already have their
 * tables allocated at declaration, so they are skipped here.
 * The value returned is "true" (success) or "false" (the required
 * table was too big to fit in the memory available).
 */

boolean set_table( functor f )
{
  int count, i;

  if (!f) return true;
  if (!set_table(f->nextfunct)) return false;
  if (strchr("=><+",f->fname[0])) return true;
  if (f->fname[0]=='E' && f->fname[1]=='!') return true;
  if (f->fname[0]=='L' && f->fname[1]=='E' && f->fname[2]=='N') return true;
  count = 1;
  for (i=0; i<f->adicity; i++) {
    if (MAXMATRIX/((f->arg_type[i]->siz)) < count) {
      //      printf("Ohhh that's way to big!\n");
      return false;
    }
    else count *= (f->arg_type[i]->siz);
  }
  if (f->table) {
    //    printf("free   %#x for `%s' %#x\n", (int)(f->table), f->fname, (int)(f));
    // fflush(stdout);
    free(f->table);
    //    printf("whahoo!\n");
    //    fflush(stdout);
  }
  f->table = (int*) malloc(sizeof(int) * count);
  f->tablesize = count; // do we need this?
  //  printf("malloc %#x (%d x int) for `%s' %#x\n", 
  //	 (int)(f->table), count, f->fname, (int)(f));
  //  fflush(stdout);
  return true;
}

// boolean set_table( functor FN )
// {
//   int count, i;
//   functor f;
// 
//   for (f=FN; f; f=f->nextfunct) {
// 
//     // but these guys never get put on FN?
//     if (strchr("=><+",f->fname[0])) continue;
//     if (f->fname[0]=='E' && f->fname[1]=='!') continue;
//     if (f->fname[0]=='L' && f->fname[1]=='E' && f->fname[2]=='N') continue;
// 
//     count = 1;
//     for (i=0; i<f->adicity; i++) {
//       if (MAXMATRIX/((f->arg_type[i]->siz)) < count) {
// 	return false;
//       }
//       else count *= (f->arg_type[i]->siz);
//     }
//     if (f->table) {
//       printf("free %x for `%s'\n", f->table, f->fname);
//       free(f->table);
//       //      printf("whahoo!\n");
//       f->table = 0;
//     }
//     f->table = (int*) malloc(sizeof(int) * count);
//     f->tablesize = count; // do we need this?
//     printf("malloc %x (%d x int) for `%s'\n", f->table, count, f->fname);  
//   }
//   return true;
// }


/*
 * Index of the part of the vector corresponding to f.
 * If f is nullary, this is just the next cell. Otherwise, the next
 * cell is the "junk" object of f, and then the vector length is
 * increased enough to allow vec_pos of f(_..._) to fit in.
 *
 * Return true iff there was enough space to do this.
 */

boolean set_up_for( functor f )
{
  int i, j;
  int argument[ARGMAX];

  if ( !f ) return true;
  if (!set_up_for( f->nextfunct )) return false;
  if ( f->piped ) return true;
  if (!f->adicity) {
    f->fcc = tr_parameters.vlength;
    tr_parameters.forbidden[tr_parameters.vlength++] = f->cut_it;
    if (tr_parameters.vlength >= V_LENGTH) return false;
  }
  else {
    f->fcc = ++tr_parameters.vlength;
    for (i=0; i<f->adicity; i++) argument[i] = 0;
    do {
      j = vec_pos(f,argument);
      if (j >= V_LENGTH) return false;
      if (j >= 0) tr_parameters.forbidden[j] = f->cut_it;
      if (j >= tr_parameters.vlength) tr_parameters.vlength = j+1;
    }
    while (another_argument(f,argument));
  }
  return true;
}



void set_sum()
{
  int i, j, k;
  functor f;
  char s[LEXLENGTH];

  for ( k = 0; sort[k].sname; k++ ) {
    for ( i = 0; i < SZ; i++ )
      for ( j = 0; j < SZ; j++ )
	sort[k].sum[i][j] = i+j < sort[k].siz? i+j: sort[k].siz;
    strcpy(s,"+_"); strcat(s,sort[k].sname);
    f = newfunctor(s,Func,2);
    if (f->val_type != sort+k) {
      for (i=0; lexicon[i].word[0]; i++) printf("%s\n",lexicon[i].word);
      inerror("Incorrectly defined addition function",s);
    }
    /*
     *    f->fats[0] = (Integer->siz)+1;
     */
    f->fats[0] = SZ;
    f->fats[1] = 1;
    sort[k].sizmask = 0;
    for ( i = 0; i < sort[k].siz; i++ )
      sort[k].sizmask |= (1 << i);
  }
}



void set_E()
{
  int i, k;

  for ( k = 0; sort[k].sname; k++ )
    for ( i = 0; i < SZ; i++ )
      sort[k].Ex[i] = i < sort[k].siz? true: false;
}



boolean set_fats(functor f)
{
  int i;

  if (f->adicity)
    f->fats[f->adicity-1] = 1;
  for (i=f->adicity-2; i>= 0; i--) {
    f->fats[i] = f->fats[i+1] * (f->arg_type[i+1]->siz);
    if (f->fats[i] * (f->arg_type[i]->siz) >= V_LENGTH) return false;
  }
  return true;
}



boolean set_cc()
{
  functor f;

  for ( f = FN; f; f = f->nextfunct ) {
    if ( f->comm && f->arg_type[0]->siz != f->arg_type[1]->siz )
      skipout("Unequal argument domains for commutative function",1);
    if (!set_fats(f)) return false;
  }
  return true;
}


void print_set( functor f )
{
    int i, j;
    int argument[ARGMAX];

    printf("\nIndex for %s",f->fname);
    if ( !f->adicity ) {
	printf(" %d\n", vec_pos(f,argument));
	return;
    }
    puts("");
    if ( f->adicity == 1 ) {
    	for ( i = 0; i < f->arg_type[0]->siz; i++ ) {
	    argument[0] = i;
	    printf("%4d",vec_pos(f,argument));
	}
	argument[0] = i;
	printf("%5d\n",vec_pos(f,argument));
	return;
    }
    for ( i = 0; i < f->arg_type[0]->siz; i++ ) {
	argument[0] = i;
	for ( j = 0; j < f->arg_type[1]->siz; j++ ) {
	    argument[1] = j;
	    printf("%4d",vec_pos(f,argument));
	}
	argument[1] = j;
	printf("%5d",vec_pos(f,argument));
	puts("");
    }
    puts("");
    argument[0] = i;
    for ( j = 0; j < f->arg_type[1]->siz; j++ ) {
	argument[1] = j;
	printf("%4d",vec_pos(f,argument));
    }
    argument[1] = j;
    printf("%5d",vec_pos(f,argument));
    puts("");
}

/*
void writeref(int x, int y, int z)
{
    int xx, yy;
    functor ff;
    char cc;

    if (z) cc = '#'; else cc = '=';
    ff = un_vec_pos(x,&xx,&yy);
    switch(ff->adicity) {
      case 0: printf("  %s%c%d", ff->fname, cc, y); break;
      case 1: printf("  %s(%d)%c%d", ff->fname, yy, cc, y); break;
      case 2: printf("  %s(%d,%d)%c%d", ff->fname, xx, yy, cc, y); break;
    }
}

*/
