/*------------------------------------------------------------------------*/
/*
 *                   Type definitions for FINDER 
 */
/*------------------------------------------------------------------------*/


typedef enum	{ None, Func, Var, Const } syntype;
/*typedef enum	{ false, true } boolean;*/
typedef enum	{ null, brief, full } prints;
typedef enum	{ nul, srt, fcn, fcn0, cls, prm, inc } statement_type;

/*--------------------- a list of cannonical names ------------------------*/
typedef struct s_name_list {
    char *name;
    struct s_name_list *nextname;
} NAME_LIST, *name_list;


/*----------------------------- a sorts -----------------------------------*/
typedef struct {
	char	*sname;		  /* The identifier for this sort          */
	int	sizmin,		  /* Least number of things of this sort   */
		sizmax,		  /* Greatest ditto			   */
		siz;		  /* Current number of such things	   */
	int	sum[SZ][SZ];	  /* Addition is a built-in function	   */
	name_list canonical;	  /* List of names, for enumerated sort	   */
	int	Ex[SZ],		  /* "Exists" is a predicate after all!	   */
		len[SZ];	  /* Name lengths of the elements          */
	boolean	piped;		  /* Sort "siz" comes via the pipe         */
	unsigned sizmask;	  /* Bit mask for range 0..siz             */
} KIND, *kind;


/*---------------------- a model of a functors ----------------------------*/
typedef struct s_functor {
    char *fname;	            /* The identifier for this symbol      */
    syntype category;	            /* Syntactic type (func, pred, etc)    */
    int scope,		            /* For parsing purposes                */
        adicity,	            /* Number of argument places           */
        *table,		            /* Start of matrix for this function   */
      tablesize,                    /* Number of ints malloced             */
      junk,                         /* Value for nonexistent arguments     */
        fcc,		            /* Index: start of portion of vector   */
        base_fcc,                   /* fcc for base model                  */
        old_fcc[MAXISET_ARRAY],     /* Was fcc on last good model          */
        fats[ARGMAX];               /* For use in matrix lookup            */
    boolean total,	            /* Function may not have null value    */
        injective,	            /* Function is one-one                 */
        leftin,		            /* Each row is one-one                 */
        rightin,	            /* Each column is one-one              */
        surjective,	            /* Function is onto                    */
        leftsur,	            /* Each row is onto                    */
        rightsur,	            /* Each column is onto                 */
        comm,		            /* Function is binary and commutative  */
        piped,		            /* Function values come via the pipe   */
        nop,		            /* No decrease in change-order priority*/
        cut_it;		            /* Prohibit backtracking on function   */
    kind arg_type[ARGMAX],          /* The types of the argument(s)        */
        val_type;	            /* The type of this function's value   */
        prints	pmode;		    /* What kind of print display          */
        struct s_functor *nextfunct;/* Linking the list of "real" functors */
} FUNCTOR, *functor;
		

/*----------------------- a well formed formula ----------------------------*/
typedef struct s_wff {
#if 0
  struct s_wff *parent;      /* The wff that this is a subformula of  */
  struct s_wff *next_arg;    /* link the args together                */
#endif 
	functor	sym;		   /* Main predicate or function symbol	    */
	struct s_wff *sb[ARGMAX];  /* Offsets of the subformulae	    */
	int	val,		   /* Value currently assigned		    */
		*matrix,	   /* Start of the relevant value table	    */
		*sbval[ARGMAX],	   /* Pointers to values of subformulae	    */
		wcc;		   /* Offset in info vector for t-testing   */
	struct s_wff *nextwff;	   /* Linking the list of testables         */
} WFF, *wff;


/*-----------------a stack of well formed formulae -------------------------*/
typedef struct s_wff_stack {
        wff	w;
        struct s_wff_stack *nxt;
} WFF_STACK, *wff_stack;


/*----------------- clause as a pair of wff lists: prem and conc -----------*/
typedef struct s_clause {
  wff_stack prem,                       /* negative literals          */
    conc,                       /* positive literals          */
    varlist,                    /* variables in the clause    */
    testlist;                   /* subformuale in test order  */
  int weight,
    priority,                   /* priority in search order   */
    varcount;                   /* number of variable         */
  boolean preprocessed;          /* has been checked for smallref */
  struct s_clause *nextclause,
    *lastclause;
} CLAUSE, *clause;


/*-------------------------- semi-parsed formulas --------------------------*/
typedef struct s_psw {
    functor main;
    wff actual;
    struct s_psw *leftsub, *rightsub;
} PSW, *psw;


/*---------------------a meaningful word in the language -------------------*/
typedef struct {
	char	word[LEXLENGTH];
	boolean	iskind;                   /* flag if word represents a sort */ 
	int	offset;                   /* its position in the lexicon    */
} LEXITEM;


/*----------------------------PARAMETERS------------------------------------*/

typedef struct {
	int	max_models,
		maxref,
		test_maxref,
		max_backtracks,
		latest_priority,
		pretest_limit,
		t_pretest_limit,
		batches,
		extra_batches;
  	logical	maxtime;
	boolean	Nosecs,
		invert_change,
		vars_over_wt,
		topsoff,
		report_breaks,
		noisy,
		timestamps;
	struct	printmode {
		unsigned verbosity;
		boolean pretty;
	}	pm;
} PARAMETERS;

/*=========================== END =========================================*/

