/*--------------------------------------------------------------*
 |  FILE: called_by_otter.c                                     |
 |                                                              |
 | SCOTT functions called OTTER only.  These exist primarily to |
 | encasulate data insulate the two programs.                   |
 *--------------------------------------------------------------*/

#define IN_SCOTT

#include "otter_header.h"     // OTTERs global constants, variables,
                              // plus prototypes
#include "defs.h"             // SCOTTs global constants
#include "shared_defs.h"      // constants shared by SCOTT, FINDER, and OTTER
#include "globs.h"            // global variables
#include "called_by_otter.h"  // protos shared by SCOTT and OTTER
#include "clocks.h"           // scott clocks

// tClocks
long scott_clock_val(int c); // prototype from scott.c

long get_sem_guide_time_val()
{
  return scott_clock_val(SEM_GUIDE_TIME);
}

long get_sem_dev_time_val()
{
  return scott_clock_val(SEM_DEV_TIME);
}
// tStats

long get_sem_res_gen_val()
{
  return (long)tStats[SEM_RES_GEN];
}

// tParms
int get_back_sub_wt_val()
{
  return tParms[BACK_SUB_WT].val;
}

// tFlags
int get_sem_res_flag()
{
  return tFlags[SEM_RES].val;
}

int get_semantic_guidance_flag()
{
  return tFlags[SEMANTIC_GUIDANCE].val;
}

int get_shuffle_sos_flag()
{
  return tFlags[SHUFFLE_SOS].val;
}

int get_unit_conflict_flag()
{
  return tFlags[UNIT_CONFLICT].val;
}

int get_auto3_flag() 
{
  return tFlags[AUTO3].val;
}

int get_dsc_flag()
{
  return tFlags[DSC].val;
}

void set_scott_clause(struct clause *p)
/*------------------------------------------------------------------*
 | Called by get_clause(). Initialises the SCOTT bits of new clause |
 *------------------------------------------------------------------*/
{
  int maxiset;

  /* initialize clause prior to testing */
  p->given = 0;
  p->length = 0;
  p->num_sets = 0;
  for (maxiset=0; maxiset < MAXISET_ARRAY; maxiset++)
    p->in_set[maxiset] = false;
  p->stype = Norm;
  p->back_subs = 0;

} /*--set_scott_clause()--*/

void scott_queue_to_level(int level)
 /*------------------------------------------------------------------*
  | Called by main(). Output stats at end of each level and check if |
  | QUEUE_TO_LEVEL has been reached.                                 |
  *------------------------------------------------------------------*/
 
{
  // this should be in a function
  output_stats(stdout, Parms[STATS_LEVEL].val);
  if (Parms[STATS_LEVEL].val != 0) output_weight_distn();
  if (tParms[QUEUE_TO_LEVEL].val && level > tParms[QUEUE_TO_LEVEL].val) {
    fprintf(stderr, "\n%cSwitching to best-first search\n\n", Bell);
    printf("\nSwitching to best-first search\n");
    Flags[SOS_QUEUE].val = false;
  }
  fflush(stdout);
} /*--scott_queue_to_level()--*/


int check_max_pick_wt(struct clause *giv_cl)
 /*----------------------------------------------------------------*
  | Called by main(). Return MAX_PICK_WT_EXIT if giv_cl is greater |
  | than MAX_PICK_WT_EXIT. handy tool when using OTTER as a        |
  | programming tasks.  Assumes clauses are chosen best-first      |
  *----------------------------------------------------------------*/
{
  
  if (giv_cl && 
      !Flags[SOS_QUEUE].val && 
      !Flags[SOS_STACK].val && 
      Parms[PICK_GIVEN_RATIO].val == -1 &&
      (giv_cl->pick_weight > tParms[MAX_PICK_WT].val))
    return MAX_PICK_WT_EXIT;
  else
    return KEEP_SEARCHING;

} /*--check_max_pick_wt()--*/
