/*========================================================\
|               FILE: called_by_otter.h                   |
|                                                         |
| Prototypes of SCOTT functions called by OTTER sources.  |
| Many of these functions reside in the file              |
| called_by_otter.c but may be dispersed thoughout the    |
| SCOTT sources.                                          |
\========================================================*/

/*--process.c---------------------------------------------*/
void scott_delete_from_base(struct clause *cl);
void scott_control_memory();
int get_sem_res_flag();
int get_dsc_flag();
boolean in_constraint_set(struct clause *cl);
void abort_scott(char *msg, ...);
int get_semantic_guidance_flag();
void add_semantic_wt(struct clause *otter_cl);
int get_shuffle_sos_flag();

/*--called from misc.c------------------------------------*/
void scott_clock_init();
void init_scott_options();
void init_distributions();
void dependent_scott_flags(FILE *fp, int index);
void dependent_scott_fparms(FILE *fp, int index);
void dependent_scott_tparms(FILE *fp, int index);
void automatic_3_settings();
int get_semantic_guidance_flag();
void init_semantics(int argc, char *argv[]);
void init_clause_space();
void init_partitions();
void output_scott_stats(FILE *fp, int level);
int get_sem_res_flag();
void sem_res(struct clause *giv_cl);
long get_sem_guide_time_val();
long get_sem_dev_time_val();
long get_sem_res_gen_val();
int get_unit_conflict_flag();

/*--called from main.c------------------------------------*/
void print_scott_banner(int argc, char **argv);
void scott_queue_to_level(int level);
void output_weight_distn();
int check_max_pick_wt(struct clause *cl);

/*--called from clause.c----------------------------------*/
void print_scott_sets(FILE *fp, struct clause *cl);
int get_back_sub_wt_val();
void scott_delete(struct list *lptr, struct clause *cl);
void scott_insert(struct list *lptr, struct clause *cl);
int get_semantic_guidance_flag();
struct clause *semantic_selection(struct list *lptr);

/*--called from options.c---------------------------------*/
void print_scott_options(FILE *fp);
int change_scott_parm(FILE *fp,
		      struct term *t,
		      char *parm_name);
int change_scott_flag(FILE *fp,
		      struct term *t,
		      char *flag_name,
		      int set);
int get_sem_res_flag();
int get_semantic_guidance_flag();
int get_dsc_flag();

/*--called from av.c--------------------------------------*/
void set_scott_clause(struct clause *p);

/*--prototypes in ../scott_dir/called_by_scott.c----------*/
long get_sem_guide_time_val();
long get_sem_dev_time_val();
long get_sem_res_gen_val();
int get_back_sub_wt_val();
int get_sem_res_flag();
int get_semantic_guidance_flag();
int get_shuffle_sos_flag();
int get_unit_conflict_flag();
int get_auto3_flag();
int get_dsc_flag();
void set_scott_clause(struct clause *p);
void scott_queue_to_level(int level);
int check_max_pick_wt(struct clause *giv_cl);
