   /*====================================\
   |           "Sdef.h":                 |
   |                                     |
   | Constants global to SCOTT sources.  |
   \====================================*/


/*---SCOTT_DEBUG()---------------------------------------------------*/
#ifdef DEBUG_SCOTT
#define SCOTT_DEBUG(event) \
{ \
    debug_partition(event); \
    debug_scan(event); \
    check_null_set_size(event); \
}
#else
#define SCOTT_DEBUG(event);
#endif
/*--------------------------------------------------------------------*/

#define MAX_DIST 20  // number of leaves in distibution leaf plots
#define NUM_POINTS 5 // maximum data points in estimation code
#define FILE_NAME_ARRAY      1024     // just in case it gets real big eg TPTP
#define FUNCTION_NAME_ARRAY  32       // probably too large

// strategy control points
#define SET             0
#define FAILS           1
#define TEST            2
#define PASS            3
#define RECORD_EVENT    4
#define RECORD_KEPT     5

/*---------------- Statistics for semantic guidance ------------------*/

#define MAX_TSTATS               40 // declared also in sfo_shared_defs.h
			         
#define ATTEMPTS                  0
#define COVER_ATTEMPTS            1
#define REFINE_ATTEMPTS           2
#define REPLACEMENT_ATTEMPTS      3 // hmm never used...
			         
#define KEPT                      4
#define COVER_KEPTS               5
#define REFINE_KEPTS              6
			         
#define FOR_SUBS                  7
#define REFINE_FAILS              8
#define SOS_COVERS                9
			         
#define BACK_SUBS                10 
#define CULLED                   11
#define CLASHES                  12 // hmm never used...
			         
#define SEM_BASE_SIZE            13
#define CONJECTURE_SIZE          14
#define BASE_CL_ADDED            15
#define BASE_CL_SKIPPED          16
#define BASE_EXTENSION_FAILS     17
			         
#define GIVENS_FROM_FIRST        18
#define CL_SEM_QUEUED            19
#define SUCCESSFUL_RETESTS       20
#define SEMANTIC_SELECTIONS      21
			         
#define FAILS_IN_PERSIST         22
#define PFAIL_TIME_INC           23
#define PFAIL_TIME_DEC           24
#define PFAIL_PROB_DEC           25
#define PFAIL_COST_VS_TIME       26
#define PFAIL_TIME_ACCRUED       27
#define PFAIL_COST_VS_RETURN     28
#define PFAIL_COST_VS_PROB       29
			          
#define MAXISETS_AT_END_DEV      30
#define SEM_RES_GEN              31
#define SOS_STRATEGY_REFINEMENTS 32

/*-- Indices for tParms[] ---------------------------------------------*/

#define MAX_TPARMS           30 // declared also in SFOshared_defs.h

#define MAX_SEARCH_TIME       0
#define MIN_SEARCH_TIME       1
#define MAX_DOMAIN_SIZE       2
#define MIN_DOMAIN_SIZE       3

#define MAX_INT_FAILS         4 // max consecutive failures to form a maxiset
                                // within a single call to semantic_selection()
#define MAX_EXT_FAILS         5 // max consecutive failures to form a maxiset
                                // accross calls to semantic_selection()
#define MAX_NUM_MAXISETS      6
#define MIN_NUM_MAXISETS      7
#define MIN_MAXISET_ATTEMPTS  8
			      
#define SEM_QUEUE_LIMIT       9 
#define MIN_REFINEMENT       10 
#define MAX_NULL_SET_SIZE    11
#define VERBOSITY            12
#define MAX_NEW_BASE_CL      13
#define TRUE_WEIGHT          14
			      
#define LIMIT_WEIGHT_TO      15
#define CULL_WEIGHTS_OVER    16
#define LIMIT_VARS_TO        17
#define CULL_VARS_OVER       18
			      
#define MEMORY_CONTROL_POINT 19
#define REVS_TO_SKIP_ON_FAIL 20 // revisons to skip after MAX_EXT_FAILS 

#define BACK_SUB_WT          21
#define QUEUE_TO_LEVEL       22
#define RANDOM_SEED          23
#define MAX_PICK_WT          24
#define SELECTION_BUFFER     25

// What about MAX_PICK_WT_EXIT?

/*-- Indices for fParms[] (parameters that take float values)---------*/

#define MAX_FPARMS          20 // declared also in SFOshared_defs.h

#define GUIDANCE_FACTOR      0
#define COST_MARGIN          1
#define MIN_PROB_EXTN        2
#define SMOOTHING_RATIO      3
#define TOLERANCE            4
#define SCALE_WEIGHT         5  
#define MIN_COST_OF_ATTEMPT  8


/*-- Indices for tFlags[] ------------------------------------------*/
#define MAX_TFLAGS          20

#define DSC                  0 // dynamic semantic constraint
#define SEM_RES              1 // (ordered) semantic resolution inference rule
#define SEMANTIC_GUIDANCE    2 // semantic selection of given clauses
#define SMOOTH_CMP_DIST      3 // selection tries to smooth dist of cmps
			     
#define AUTO3                4 // and a new autonomous mode
#define UNIT_CONFLICT        5 // so we can turn it off if we want
#define SHUFFLE_SOS          6 // insert clauses randomly in SOS
#define AUTO_SET_CONJECTURE  7 // check that a good conjecture is set
#define AUTO_SET_SOS         8 // optimise SOS strategy

#define DEMOD_IN_CONJECTURE  9 // place all demodualtors in conjecture list
#define DEMOD_IN_SEM_BASE   10 // place all demodualtors in semantic base
#define CONJ_IN_INPUT_FIRST 11 // place conj clauses on initial input first
#define USABLE_IN_SEM_BASE  12 // place initial USABLE in semantic base

#define MODEL_USABLE_LIST   12 // attempt to model USABLE clauses during scans

/*-- Indices for tClocks[] ------------------------------------------*/
#define MAX_TCLOCKS         5

#define SEM_RES_TIME        0
#define SEM_DEV_TIME        1
#define SEM_GUIDE_TIME      2

/*--History event definitions---------------------------------------*/

#define STRENGTHEN_EQ_DEF	"\n\nstrengthen_eq_def\n"
#define VERIFY_MODEL		"\n\nverify model\n"

#define SELECT_LIGHTEST		"->l..."
#define SELECT_BEST_SEM		"->s..."
#define START_INPUT_FIRST	"->f..."
#define START_SEM_BREADTH	"->b..."
#define SELECT_QUEUE            "Q"

#define DISCARD_BAD_MAXISET	"B..."
#define COVER_NEW_CLAUSES	"C"
#define REFINE_CHOICE	        "R"

#define DISCARD_NEW_BASE_CL	"[dbase]"
#define STRENGTHEN_BASE_FAILURE "[fbase]"

#define FOR_SUB_MAXISET		"[fsub]"
#define BACK_SUB_MAXISET	"[bsub]"
#define MAXISET_CULLED		"[cull]"
#define FAILS_TO_REFINE	        "[ref]"
#define COVERS_SOS              "[sos]"

