   /*=============================================\
   |               FILE: globs.h                  |
   |                                              |
   | External variables shared by SCOTT sources.  |
   \=============================================*/
   
#ifdef IN_SCOTT
#define SGLOBAL
#else
#define SGLOBAL extern				
#endif

typedef int (*int_function)();

SGLOBAL struct {  /* fParms are real valued options */
  char *name;
  double val;
  double min, max;  /* minimum and maximum permissible values */
} fParms[MAX_FPARMS];

SGLOBAL struct {  /* tParms are integer valued options */
  char *name;
  int val;
  int min, max;  /* minimum and maximum permissible values */
} tParms[MAX_TPARMS];

SGLOBAL struct {  /* tFlags are integer (boolean) valued options */
  char *name;
  int val;
} tFlags[MAX_TFLAGS];

SGLOBAL int tStats[MAX_TSTATS];

SGLOBAL struct scott_clock {   /* for timing used by semantic guidance strategy */
  long accum_sec;        /* accumulated time */
  long accum_usec;
  long curr_sec;         /* time since clock has been turned on */
  long curr_usec;
} tClocks[MAX_TCLOCKS];

/*--------------------------------------------------------------.
| The clause space can be partitioned according to the number   |
| of sets (NMCSs or co-NMCSs) each clause belongs to.  This     |
| partitioning can be further broken down into list membership  |
| (usable, sos) and set membership. The size of each cell in    |
| this partitions is stored in the following "partition" data   |
| structures.  For example if "foo_sets" is a partition then    |
| foo_sets.sos[N][K] is the number of sos cluases in the "foo"  |
| set N that also belongs to K many "foo" sets. The set number  |
| 0 is used to store the partition without this break down:     |
| foo_sets.sos[0][K] is simply the number of sos clauses that   |
| belong to K many "foo" sets.  Since a clause cannot belong to |
| a particular set and also belong to 0 many sets, position 0   |
| is used to store the total size of that set, for example,     |
| foo_sets.sos[N][0] is the number of sos clauses in "foo" set  |
| N.                                                            |
`--------------------------------------------------------------*/
struct partition {
  int sos[MAXISET_ARRAY][MAXISET_ARRAY];
  int usable[MAXISET_ARRAY][MAXISET_ARRAY];
};

struct clause_dptr {  /*a doubly linked list of clause pointers */
  struct clause *c;
  struct clause_dptr *prev, *next;
};

struct external_list {
  struct clause_dptr *first_cp, *last_cp;
};

struct data_array {
  char name[20];
  int num_points;
  float total;
  float list[NUM_POINTS] ;
};

struct Distn {
  int min,max;
  int freq[MAX_DIST];
  int freq_over_comp[MAXISET_ARRAY][MAX_DIST];

  int max_outlier;
  int num_outliers_up;
  int outliers_up_over_comp[MAXISET_ARRAY];

  int min_outlier;
  int num_outliers_down;
  int outliers_down_over_comp[MAXISET_ARRAY];

  float average, median, variance;
  int interval;
  char *name;
  int (*property)();
  int *limit;
  int *cull_over;
  int *limit_at;
};

SGLOBAL struct external_list conjecture;
SGLOBAL struct external_list old_conjecture;
SGLOBAL struct external_list semantic_base;
SGLOBAL struct external_list sos_scan[MAXISET_ARRAY];
SGLOBAL struct external_list failed_clauses[MAXISET_ARRAY];
SGLOBAL struct external_list new_base_clauses;

// these are shared and referenced often enough
SGLOBAL int last_queue_theory;
SGLOBAL int last_set_index;         // largest availiable maxiset index
SGLOBAL int num_maxisets;         // number of maxisets in the current rose
SGLOBAL int constraint_set;       // maxiset used for semantic resolution
SGLOBAL boolean active[MAXISET_ARRAY]; // satus of maxisets index's
                                       // note the zero theory is always active
SGLOBAL int null_set_size;
SGLOBAL boolean develop_semantics;
SGLOBAL boolean recording;
