  /*===========================================================\
  |                     FILE: options.c                        |
  |                                                            |
  | Functions for managing parameters local to SCOTT sources.  |
  \===========================================================*/

#include "scott.h"

void init_scott_options()
  /*----------------------------------------------------------------*
   | Called by OTTER fcn init().  Initialise options local to SCOTT |
   | sources.                                                       |
   *----------------------------------------------------------------*/
{
  int parm, flag;
  
  // would prefer to use enumerated types here or some thing else here
  for (flag = 0; flag < MAX_TFLAGS; flag++)
    tFlags[flag].name = "";

  for (parm = 0; parm < MAX_TPARMS; parm++)    
    tParms[parm].name = "";
  
  for (parm = 0; parm < MAX_FPARMS; parm++)
    fParms[parm].name = "";

  tFlags[AUTO3].name = "auto3";
  tFlags[AUTO3].val = 0;
  
  tFlags[DSC].name = "dsc";
  tFlags[DSC].val = 0;
  
  tFlags[SHUFFLE_SOS].name = "shuffle_sos";
  tFlags[SHUFFLE_SOS].val = 0;
  
  tFlags[UNIT_CONFLICT].name = "unit_conflict"; 
  tFlags[UNIT_CONFLICT].val = 1; // default is set
  
  tFlags[SEM_RES].name = "semantic_res"; 
  tFlags[SEM_RES].val = 0;
  
  tFlags[SEMANTIC_GUIDANCE].name = "semantic_guidance"; 
  tFlags[SEMANTIC_GUIDANCE].val = 0;
  
  tFlags[SMOOTH_CMP_DIST].name = "smooth_cmp_dist"; 
  tFlags[SMOOTH_CMP_DIST].val = 0;
  
  tFlags[AUTO_SET_CONJECTURE].name = "auto_set_conjecture";
  tFlags[AUTO_SET_CONJECTURE].val = 0;
  
  tFlags[AUTO_SET_SOS].name = "auto_set_sos";
  tFlags[AUTO_SET_SOS].val = 0;

  tFlags[USABLE_IN_SEM_BASE].name = "usable_in_sem_base";
  tFlags[USABLE_IN_SEM_BASE].val = 1; // default is set

  tFlags[DEMOD_IN_CONJECTURE].name = "demod_in_conjecture";
  tFlags[DEMOD_IN_CONJECTURE].val = 0; // default is clear

  tFlags[DEMOD_IN_SEM_BASE].name = "demod_in_sem_base";
  tFlags[DEMOD_IN_SEM_BASE].val = 1;   // default is set

  tFlags[CONJ_IN_INPUT_FIRST].name = "conj_in_input_first";
  tFlags[CONJ_IN_INPUT_FIRST].val = 0;   // default is clear
  
  tFlags[MODEL_USABLE_LIST].name = "model_usable_list";
  tFlags[MODEL_USABLE_LIST].val  = 1; // default is on

  tParms[MAX_PICK_WT].name = "max_pick_wt";
  tParms[MAX_PICK_WT].val = MAX_INT;
  tParms[MAX_PICK_WT].min = -MAX_INT;
  tParms[MAX_PICK_WT].max = MAX_INT;
  
  tParms[RANDOM_SEED].name ="random_seed";
  tParms[RANDOM_SEED].val = 0;
  tParms[RANDOM_SEED].min = 0;
  tParms[RANDOM_SEED].max = MAX_INT;
  
  tParms[QUEUE_TO_LEVEL].name ="queue_to_level";
  tParms[QUEUE_TO_LEVEL].val = 0;
  tParms[QUEUE_TO_LEVEL].min = 0;
  tParms[QUEUE_TO_LEVEL].max = MAX_INT;
  
  tParms[BACK_SUB_WT].name = "back_sub_wt";
  tParms[BACK_SUB_WT].val = 0;
  tParms[BACK_SUB_WT].min = -MAX_INT;
  tParms[BACK_SUB_WT].max = MAX_INT;

  tParms[SELECTION_BUFFER].name = "selection_buffer";
  tParms[SELECTION_BUFFER].val = 10;
  tParms[SELECTION_BUFFER].min = 0;
  tParms[SELECTION_BUFFER].max = MAX_INT;

  tParms[MEMORY_CONTROL_POINT].name = "memory_control_point";
  tParms[MEMORY_CONTROL_POINT].val = 0; // off by default
  tParms[MEMORY_CONTROL_POINT].min = 0;
  tParms[MEMORY_CONTROL_POINT].max = MAX_INT;

  tParms[LIMIT_WEIGHT_TO].name = "limit_weight_to";
  tParms[LIMIT_WEIGHT_TO].val = 10; // percentile
  tParms[LIMIT_WEIGHT_TO].min = 0;
  tParms[LIMIT_WEIGHT_TO].max = 100;
  
  tParms[CULL_WEIGHTS_OVER].name = "cull_weights_over";
  tParms[CULL_WEIGHTS_OVER].val = 90;  // percentile
  tParms[CULL_WEIGHTS_OVER].min = 0;
  tParms[CULL_WEIGHTS_OVER].max = 100;

  tParms[LIMIT_VARS_TO].name = "limit_vars_to";
  tParms[LIMIT_VARS_TO].val = 50; // percentile
  tParms[LIMIT_VARS_TO].min = 0;
  tParms[LIMIT_VARS_TO].max = 100;
  
  tParms[CULL_VARS_OVER].name = "cull_vars_over";
  tParms[CULL_VARS_OVER].val = 90;  // percentile
  tParms[CULL_VARS_OVER].min = 0;
  tParms[CULL_VARS_OVER].max = 100;

  tParms[MAX_DOMAIN_SIZE].name = "max_domain_size";
  tParms[MAX_DOMAIN_SIZE].val = 10;
  tParms[MAX_DOMAIN_SIZE].min = 1;
  tParms[MAX_DOMAIN_SIZE].max = MAX_INT;

  tParms[MIN_DOMAIN_SIZE].name = "min_domain_size";
  tParms[MIN_DOMAIN_SIZE].val = 0;
  tParms[MIN_DOMAIN_SIZE].min = 0;
  tParms[MIN_DOMAIN_SIZE].max = MAX_INT;

  tParms[MAX_NUM_MAXISETS].name ="max_num_maxisets";
  tParms[MAX_NUM_MAXISETS].val = 5;
  tParms[MAX_NUM_MAXISETS].min = 0;
  tParms[MAX_NUM_MAXISETS].max = MAXISET_ARRAY-2;
  
  tParms[MIN_NUM_MAXISETS].name ="min_num_maxisets";
  tParms[MIN_NUM_MAXISETS].val = 3;
  tParms[MIN_NUM_MAXISETS].min = 0;
  tParms[MIN_NUM_MAXISETS].max = MAXISET_ARRAY-2;
  
  tParms[MIN_MAXISET_ATTEMPTS].name ="min_maxiset_attempts";
  tParms[MIN_MAXISET_ATTEMPTS].val = 5;
  tParms[MIN_MAXISET_ATTEMPTS].min = 1;
  tParms[MIN_MAXISET_ATTEMPTS].max = MAXISET_ARRAY-2;
  
  tParms[MAX_SEARCH_TIME].name = "max_search_time";
  tParms[MAX_SEARCH_TIME].val = 2000;  // millisecs
  tParms[MAX_SEARCH_TIME].min = 0;
  tParms[MAX_SEARCH_TIME].max = MAX_INT;
  
  tParms[MIN_SEARCH_TIME].name = "min_search_time";
  tParms[MIN_SEARCH_TIME].val = 500;   // millisecs
  tParms[MIN_SEARCH_TIME].min = 0;
  tParms[MIN_SEARCH_TIME].max = MAX_INT;
  
  tParms[TRUE_WEIGHT].name ="true_weight";
  tParms[TRUE_WEIGHT].val = 0;
  tParms[TRUE_WEIGHT].min = -MAX_INT;
  tParms[TRUE_WEIGHT].max = MAX_INT;
  
  tParms[VERBOSITY].name = "verbosity";
  tParms[VERBOSITY].val = 1;
  tParms[VERBOSITY].min = 0;
  tParms[VERBOSITY].max = 5;
  
  tParms[MAX_NEW_BASE_CL].name = "max_new_base_cl";
  tParms[MAX_NEW_BASE_CL].val = 20;
  tParms[MAX_NEW_BASE_CL].min = 0;
  tParms[MAX_NEW_BASE_CL].max = 100;
  
  tParms[SEM_QUEUE_LIMIT].name = "sem_queue_limit";
  tParms[SEM_QUEUE_LIMIT].val = 2;
  tParms[SEM_QUEUE_LIMIT].min = 0;
  tParms[SEM_QUEUE_LIMIT].max = MAX_INT;
  
  tParms[MIN_REFINEMENT].name = "min_refinement";
  tParms[MIN_REFINEMENT].val = 0;
  tParms[MIN_REFINEMENT].min = -MAX_INT;
  tParms[MIN_REFINEMENT].max = 0;
  
  tParms[MAX_INT_FAILS].name ="max_int_fails";
  tParms[MAX_INT_FAILS].val = 2;
  tParms[MAX_INT_FAILS].min = 0; // then feature is OFF
  tParms[MAX_INT_FAILS].max = MAX_INT;

  tParms[MAX_EXT_FAILS].name ="max_ext_fails";
  tParms[MAX_EXT_FAILS].val = 0;
  tParms[MAX_EXT_FAILS].min = 0; // then feature is OFF
  tParms[MAX_EXT_FAILS].max = MAX_INT;

  tParms[REVS_TO_SKIP_ON_FAIL].name ="revs_to_skip_on_fail";
  tParms[REVS_TO_SKIP_ON_FAIL].val = 0;
  tParms[REVS_TO_SKIP_ON_FAIL].min = 0; // then feature is OFF
  tParms[REVS_TO_SKIP_ON_FAIL].max = MAX_INT;

  tParms[MAX_NULL_SET_SIZE].name ="max_null_set_size";
  tParms[MAX_NULL_SET_SIZE].val = 5;
  tParms[MAX_NULL_SET_SIZE].min = 0;
  tParms[MAX_NULL_SET_SIZE].max = MAX_INT;
  
  // float parameters
  fParms[MIN_PROB_EXTN].name = "min_prob_extn";
  fParms[MIN_PROB_EXTN].val = 0.3;
  fParms[MIN_PROB_EXTN].min = 0;
  fParms[MIN_PROB_EXTN].max = 1;
  
  fParms[SCALE_WEIGHT].name ="scale_weight";
  fParms[SCALE_WEIGHT].val = 0;
  fParms[SCALE_WEIGHT].min = 0;
  fParms[SCALE_WEIGHT].max = MAX_INT;
  
  fParms[TOLERANCE].name ="tolerance";
  fParms[TOLERANCE].val = 1.5;
  fParms[TOLERANCE].min = 1;
  fParms[TOLERANCE].max = MAX_INT;
  
  fParms[SMOOTHING_RATIO].name ="smoothing_ratio";
  fParms[SMOOTHING_RATIO].val = 0.225;
  fParms[SMOOTHING_RATIO].min = 0;
  fParms[SMOOTHING_RATIO].max = 1;
  
  fParms[GUIDANCE_FACTOR].name ="guidance_factor";
  fParms[GUIDANCE_FACTOR].val = 5;
  fParms[GUIDANCE_FACTOR].min = 0;
  fParms[GUIDANCE_FACTOR].max = MAX_INT;
  
  fParms[COST_MARGIN].name ="cost_margin";
  fParms[COST_MARGIN].val = 10;
  fParms[COST_MARGIN].min = 0;
  fParms[COST_MARGIN].max = MAX_INT;
  
  fParms[MIN_COST_OF_ATTEMPT].name ="min_cost_of_attempt";
  fParms[MIN_COST_OF_ATTEMPT].val = 2.0;  // seconds
  fParms[MIN_COST_OF_ATTEMPT].min = 0.0;
  fParms[MIN_COST_OF_ATTEMPT].max = MAX_INT;
  
}/*--init_scott_options()--*/


int change_scott_flag(FILE *fp, struct term *t, char *flag_name, int set)
/*-----------------------------------------------------------------------*
 |               Called by OTTER fcn change_flag().                      |
 |                                                                       |
 | Checks for parameter from amongst those local to SCOTT sources. If    |
 | found, returns its id plus 1000, if in error returns -1, else returns |
 | the value -2 which signals change_flag() to continue looking for a    |
 | match.                                                                |
 *-----------------------------------------------------------------------*/

{
  int index;
  
  for (index = 0; 
       tFlags[index].name && index < MAX_TFLAGS; 
       index++) {

    if (str_ident(flag_name, tFlags[index].name)) {
      
      if (tFlags[index].val == set) {
	fprintf(fp, "WARNING: ");
	print_term(fp, t);
	if (set)
	  fprintf(fp, " flag already set.\n");
	else
	  fprintf(fp, " flag already clear.\n");
	return(index+1000);
      }
      else {
	tFlags[index].val = set;
	if (index == BELL)
	  Bell = set ? '\007' : '\000';
	return(index+1000);
      }
    }
  }
  return -2;
  
} /*--change_scott_flag()--*/

void auto_change_scott_flag(FILE *fp, int index, int val)
  /*--------------------------------------------------------------------*
   |       Mutually recursive with dependent_scott_flags().             |
   |                                                                    |
   | Toggle the value of tFlags[index] iff it is ifferent from val, and |
   | propagate any implications of this change.                                |
   *--------------------------------------------------------------------*/
{
  if (tFlags[index].val != val) {
    fprintf(fp, "   dependent: %s(%s).\n",
	    val ? "set" : "clear", tFlags[index].name);
    tFlags[index].val = val;
    dependent_scott_flags(fp, index);
  }
} /*--auto_change_scott_flag()--*/


void dependent_scott_flags(FILE *fp, int index)
  /*-----------------------------------------------------------------------*
   |              Called initially by read_all_input()                     |
   |                                                                       |
   |          Mutually recursive with dependent_scott_flags().             |
   |                                                                       |
   | Reset flags that are dependent on the recently changed tFlag[index].  |
   *-----------------------------------------------------------------------*/
{
  int count;
  
  if (tFlags[index].val) {
    switch (index) {
    case AUTO3:  /* for SCOTT v5.1.2 */
      auto_change_flag(fp, PRINT_KEPT, 0);
      auto_change_flag(fp, PRINT_NEW_DEMOD, 0);
      auto_change_flag(fp, PRINT_BACK_DEMOD, 0);
      auto_change_flag(fp, PRINT_BACK_SUB, 0);
      auto_change_flag(fp, PROCESS_INPUT, 1);
      auto_change_parm(fp, PICK_GIVEN_RATIO, 4);
      auto_change_parm(fp, STATS_LEVEL, 1);
      auto_change_parm(fp, MAX_SECONDS, 10800);
    
      auto_change_scott_flag(fp, SEMANTIC_GUIDANCE, 1);
      auto_change_scott_flag(fp, SMOOTH_CMP_DIST, 1);
      auto_change_scott_flag(fp, AUTO_SET_CONJECTURE, 1);
      auto_change_scott_flag(fp, AUTO_SET_SOS, 1);
    
      auto_change_flag(fp, CONTROL_MEMORY, 1);
      auto_change_parm(fp, MAX_DISTINCT_VARS, 20);
      auto_change_parm(fp, MAX_WEIGHT, 200);

      auto_change_scott_tparm(fp, MEMORY_CONTROL_POINT,15000);
      auto_change_scott_tparm(fp, LIMIT_WEIGHT_TO, 30);
      auto_change_scott_tparm(fp, CULL_WEIGHTS_OVER, 90);
      auto_change_scott_tparm(fp, LIMIT_VARS_TO, 50);
      auto_change_scott_tparm(fp, CULL_VARS_OVER, 90);
    			     	  
      auto_change_scott_tparm(fp, MAX_DOMAIN_SIZE, 10);
      auto_change_scott_tparm(fp, MIN_DOMAIN_SIZE, 0);
      auto_change_scott_tparm(fp, MAX_NUM_MAXISETS, 5);
      auto_change_scott_tparm(fp, MIN_NUM_MAXISETS, 3);
      auto_change_scott_tparm(fp, MIN_MAXISET_ATTEMPTS,5);
      auto_change_scott_tparm(fp, MAX_SEARCH_TIME, 2000);
      auto_change_scott_tparm(fp, MIN_SEARCH_TIME, 500);
      auto_change_scott_tparm(fp, SEM_QUEUE_LIMIT, 2);
      auto_change_scott_tparm(fp, VERBOSITY, 1);
    
      /* other options are set after clauses are read */
      break;
    case SEM_RES:
      // sem_res needs the semantic guidance machinery
      auto_change_scott_flag(fp, SEMANTIC_GUIDANCE, 1);
      break;

      // these two flags are mutually exclusive!!!
    case DEMOD_IN_SEM_BASE:
      auto_change_scott_flag(fp, DEMOD_IN_CONJECTURE, 0);
      break;
    case DEMOD_IN_CONJECTURE:
      auto_change_scott_flag(fp, DEMOD_IN_SEM_BASE, 0);
      break;

      // not really a flag but this seems appropriate
    case SHUFFLE_SOS:
      // seed the random number generator
      if (tParms[RANDOM_SEED].val) srand(tParms[RANDOM_SEED].val);
      // skip the fuzz in the initial segment of random distribution 
      for (count = 0; count < 500; count++) rand();
      break;
    }   
  }
}  /*--dependent_scott_flags()--*/


void auto_change_scott_tparm(FILE *fp, int index, int val)
  /*---------------------------------------------------------------------*
   |       Mutually recursive with dependent_scott_tparms().             |
   |                                                                     |
   | Toggle the value of tParms[index] iff it is different from val, and |
   | propagate any implications of this change.                          |
   *---------------------------------------------------------------------*/
{
  if (tParms[index].val != val) {
    fprintf(fp, "   dependent: assign(%s, %d).\n",
	    tParms[index].name, val);
    
    tParms[index].val = val;
    dependent_scott_tparms(fp, index);
  }
}/*--auto_change_scott_tparm()--*/


void dependent_scott_tparms(FILE *fp, int index)
  /*-------------------------------------------------------------*
   |              Called initially by read_all_input()           |
   |                                                             |
   |          Mutually recursive with dependent_scott_tparms().  |
   |                                                             |
   | Reset flags that are dependent on the recently changed      |
   | tParms[index].                                              |
   *-------------------------------------------------------------*/
{
  if (tParms[index].val) {
    switch (index) {
    case QUEUE_TO_LEVEL:
      auto_change_flag(fp, SOS_QUEUE, 1);
      break;
    case MAX_PICK_WT:
      auto_change_flag(fp, SOS_QUEUE, 0);
      auto_change_flag(fp, SOS_STACK, 0);
      auto_change_parm(fp, PICK_GIVEN_RATIO, -1);
      break;
    }   
  }
}  /*--dependent_scott_tparms()--*/


void auto_change_scott_fparm(FILE *fp, int index, float val)
  /*---------------------------------------------------------------------*
   |       Mutually recursive with dependent_scott_fparms().             |
   |                                                                     |
   | Toggle the value of fParms[index] iff it is different from val, and |
   | propagate any implications of this change.                          |
   *---------------------------------------------------------------------*/
{
  if (fParms[index].val != val) {
    fprintf(fp, "   dependent: assign(%s, %f).\n",
	    fParms[index].name, val);
    
    fParms[index].val = val;
    dependent_scott_fparms(fp, index);
  }
}  /*--auto_change_scott_fparm()--*/


void dependent_scott_fparms(FILE *fp, int index)
 /*------------------------------------------------------------------------*
  |              Called initially by read_all_input()                      |
  |                                                                        |
  |          Mutually recursive with dependent_scott_fparms().             |
  |                                                                        |
  | Reset flags that are dependent on the recently changed fParms[index].  |
  *------------------------------------------------------------------------*/
{
  if (fParms[index].val) {
    switch (index) {
      // none as yet:-)
    }   
  }
}  /*--dependent_scott_fparms()--*/


int change_scott_parm(FILE *fp, struct term *t, char *parm_name)
 /*-----------------------------------------------------------------------*
  | Called by OTTER fcn change_parm().                                    |
  |                                                                       |
  | Checks for parameter from amongst those local to SCOTT sources. If    |
  | found, returns its id (plus 1000 if a tParm[] or 2000 if an fParm[]), |
  | if in error returns -1, else returns -2 which signals change_flag()   |
  | to continue looking for a match.                                      |
  *-----------------------------------------------------------------------*/
{
  int index, rc;
  char *int_name;

  // might be an integer valued parameter
  for (index = 0; 
       tParms[index].name && index < MAX_TPARMS; 
       index++) {
    int new_val;
    
    if (str_ident(parm_name, tParms[index].name)) {
      
      int_name = sn_to_str(t->farg->narg->argval->sym_num);
      rc = str_int(int_name, &new_val);
      if (rc == 0) {
	fprintf(fp, "ERROR: ");
	print_term(fp, t);
	fprintf(fp, " second argument must be an integer.\n");
	Stats[INPUT_ERRORS]++;
	return(-1);
      }
      
      if (new_val < tParms[index].min || 
	  new_val > tParms[index].max) {
	fprintf(fp, "ERROR: ");
	print_term(fp, t);
	fprintf(fp, " integer must be in range [%d,%d].\n",
		tParms[index].min, tParms[index].max);
	Stats[INPUT_ERRORS]++;
	return(-1);
      }
      
      if (new_val == tParms[index].val) {
	fprintf(fp, "WARNING: ");
	print_term(fp, t);
	fprintf(fp, " already has that value.\n");
	return(index);
      }
      tParms[index].val = new_val;
      
      return(index+1000); // so we know its a tParm[]
    }
  }

  // might be a float valued parameter.
  for (index = 0; 
       fParms[index].name && index < MAX_FPARMS; 
       index++) {
    double new_fval;
    
    if (str_ident(parm_name, fParms[index].name)) {
      
      int_name = sn_to_str(t->farg->narg->argval->sym_num);
      rc = str_double(int_name, &new_fval);
      if (rc == 0) {
	fprintf(fp, "ERROR: ");
	print_term(fp, t);
	fprintf(fp, " second argument must be a float.\n");
	Stats[INPUT_ERRORS]++;
	return(-1);
      }
      
      if (new_fval < fParms[index].min || 
	  new_fval > fParms[index].max) {
	fprintf(fp, "ERROR: ");
	print_term(fp, t);
	fprintf(fp, " float must be in range [%.2f,%.2f].\n",
		fParms[index].min, fParms[index].max);
	Stats[INPUT_ERRORS]++;
	return(-1);
      }
      
      if (new_fval == fParms[index].val) {
	fprintf(fp, "WARNING: ");
	print_term(fp, t);
	fprintf(fp, " already has that value.\n");
	return(index);
      }
      fParms[index].val = new_fval;
      
      return(index+2000); // so we know its a fParm[]
    }
  }

  return -2;

}/*--change_scott_parm()--*/


void print_scott_options(FILE *fp)
  /*----------------------------------------------------------------------*
   | Called by OTTER function print_options()                             |
   |                                                                      |
   | This esentially debugging code which allows us to check that options |
   | are being set properly.                                              |
   *----------------------------------------------------------------------*/
{
    int index, count;

    fprintf(fp, "\n---------------SCOTT options ---------------\n");

    count = 0;
    for (index = 0; index < MAX_TFLAGS; index++)  /* print set flags */
	if (tFlags[index].name[0] != '\0') {
            fprintf(fp, "%s", tFlags[index].val ? "set(" : "clear(");
	    fprintf(fp, "%s). ", tFlags[index].name);
	    count++;
	    if (count % 3 == 0)
	        fprintf(fp, "\n");
            }

    fprintf(fp, "\n\n");

    count = 0;
    for (index = 0; index < MAX_TPARMS; index++)  /* print parms */
	if (tParms[index].name[0] != '\0') {
	    fprintf(fp, "assign(");
	    fprintf(fp, "%s, %d). ", tParms[index].name, tParms[index].val);
	    count++;
	    if (count % 3 == 0)
		fprintf(fp, "\n");
	    }
    fprintf(fp, "\n");

    count = 0;
    for (index = 0; index < MAX_FPARMS; index++)  /* print parms */
	if (fParms[index].name[0] != '\0') {
	    fprintf(fp, "assign(");
	    fprintf(fp, "%s, %f). ", fParms[index].name, fParms[index].val);
	    count++;
	    if (count % 3 == 0)
		fprintf(fp, "\n");
	    }
    fprintf(fp, "\n");

}  /* print_options */
