/*--prototypes in scott_dir/auto.c--*/

boolean all_clauses(int (*property)());
boolean at_least_one_clause(int (*property)());
int get_max(int (*property)());
int get_min(int (*property)());
int get_num(int (*property)());
int clause_weight(struct clause *c);
int any_clause(struct clause *c);
int nonground_clause(struct clause *c);
int mix_clause(struct clause *c);
int mix_ground_clause(struct clause *c);
int mix_nonground_clause(struct clause *c);
int pos_ground_clause(struct clause *c);
int pos_ground_unit_clause(struct clause *c);
int pos_ground_nonunit_clause(struct clause *c);
int pos_nonground_clause(struct clause *c);
int pos_nonground_unit_clause(struct clause *c);
int pos_nonground_nonunit_clause(struct clause *c);
int neg_ground_clause(struct clause *c);
int neg_ground_unit_clause(struct clause *c);
int neg_ground_nonunit_clause(struct clause *c);
int neg_nonground_clause(struct clause *c);
int neg_nonground_unit_clause(struct clause *c);
int neg_nonground_nonunit_clause(struct clause *c);
int equality_axiom(struct clause *c);
boolean has_pos_nonground(struct list *lptr);

boolean has_pos_nonground(struct list *lptr);
boolean has_neg_nonground(struct list *lptr);

boolean has_neg_nonground(struct list *lptr);
void automatic_3_settings();
void auto_set_sos();
int_function auto_set_conjecture();
int_function rare_clause_type(char *type_message);
void set_properties();
void check_eq_settings();


/*--prototypes in scott_dir/called_by_otter.c--*/

long scott_clock_val(int c); // prototype from scott.c

long get_sem_guide_time_val();
long get_sem_dev_time_val();
long get_sem_res_gen_val();
int get_back_sub_wt_val();
int get_sem_res_flag();
int get_semantic_guidance_flag();
int get_shuffle_sos_flag();
int get_unit_conflict_flag();
int get_auto3_flag();
int get_dsc_flag();
void set_scott_clause(struct clause *p);
void scott_queue_to_level(int level);
int check_max_pick_wt(struct clause *giv_cl);


/*--prototypes in scott_dir/clause_space.c--*/

void init_clause_dptr();
struct clause_dptr *get_clause_dptr();
void free_clause_dptr(struct clause_dptr *p);
void delete_cl_ext(struct clause *c, struct external_list *elist);
void prepend_cl_ext(struct clause *c, struct external_list *elist);
void append_cl_ext(struct clause *c, struct external_list *elist);
void sos_scan_insert(struct clause *c);
void sos_scan_delete(struct clause *c);
void declare_soft_equality();
void declare_hard_equality();
void init_clause_space();
void init_conjecture_list();
void init_semantic_base();
void check_conjecture_list();
boolean make_finder_clause(char *finder_clause, struct clause *otter_clause);
boolean add_finder_term(char *finder_clause, struct term *otter_term);
void in_set_to_str(char *in_set, struct clause *cl);
boolean in_set_identical(struct clause *c1, struct clause *c2);
void print_sos_scan();
void print_failed_clause_nos();
void print_conjecture_nos();
void print_base_nos();
void print_ext_list(struct external_list *elist);
void debug_scan(char *event);
void print_sos();
void print_base();
void print_new_base();
void print_conjecture();
void print_usable();
void print_first();


/*--prototypes in scott_dir/control.c--*/

void scott_control_memory();
void control_distn(struct Distn *distn);
void cull_clauses_over(int max_value, int (*property)());
int weight_percentile(int percent);
int variable_percentile(int percent);
int clause_pick_weight(struct clause *cl);
void init_distributions();
void init_a_distribution(struct Distn *distn);
void get_distn(struct Distn *distn);
void set_distn(struct Distn *distn);
void output_distn(struct Distn *distn);
void output_weight_distn();
void output_variable_distn();
int percentile(struct Distn *distn, int percent);


/*--prototypes in scott_dir/cost.c--*/

void init_costs();
void reset_prob_of_success();
void set_data_array(struct data_array *array, float value);
void set_cost_of_given(float value);
void set_cost_per_failed_clause(float value);
void set_prob_of_success(float value);
void set_prop_failed_clauses(float value);
void set_time_saved(int givens_avoided);
void set_change_in_time_saved();
void data_list_to_str(char *array_str, struct data_array *array);
float est_from_data_array(struct data_array *array);
float est_cost_per_failed_clause();
float est_prob_of_success();
float est_change_in_time_saved();
float gradient(struct data_array *array);
float intercept(struct data_array *array);
boolean strictly_increasing(struct data_array *array);
boolean strictly_decreasing(struct data_array *array);
boolean decreasing(struct data_array *array);
void end_development(char *event);

void end_development(char *event);
boolean persist();


/*--prototypes in scott_dir/declare_symbols.c--*/

boolean insert_predicate(int sym_num);
boolean insert_function(int sym_num);
boolean insert_constant(int sym_num);
boolean scan_term_for_symbols(struct term *subterm);
boolean scan_list_for_symbols(struct list *list_ptr);
boolean scan_lists_for_symbols();
void debug_print_symbols();
boolean declare_otter_symbols();


/*--prototypes in scott_dir/options.c--*/

void init_scott_options();
int change_scott_flag(FILE *fp, struct term *t, char *flag_name, int set);
void auto_change_scott_flag(FILE *fp, int index, int val);
void dependent_scott_flags(FILE *fp, int index);
void auto_change_scott_tparm(FILE *fp, int index, int val);
void dependent_scott_tparms(FILE *fp, int index);
void auto_change_scott_fparm(FILE *fp, int index, float val);
void dependent_scott_fparms(FILE *fp, int index);
int change_scott_parm(FILE *fp, struct term *t, char *parm_name);
void print_scott_options(FILE *fp);


/*--prototypes in scott_dir/partition.c--*/

int get_usable_partition(int maxiset);
int get_sos_partition(int maxiset);
int get_size_on_sos(int maxiset);
int get_comp_on_sos(int maxiset);
int get_comp_on_usable(int maxiset);
int get_size_on_usable(int maxiset);
int get_comp_size(int maxiset);
int num_failed_clauses();
void print_a_partition(struct partition *part_ptr);
void print_partition(char *event);
boolean subset(int set1, int set2);
boolean sos_subset(int set1, int set2);
boolean already_exists(int new_maxiset);
int superset_of(int new_maxiset);
boolean verify_model_in_list(int maxiset, struct list *lptr, char list_name[]);
void check_for_model(int new_maxiset);
float size_choice_space();
float rate_partition();
float rate_maxiset(int skip_set);
float rate_clause(struct clause *cl);
void set_sem_queue(struct list *lptr);
int next_set_to_skip();
int variance(struct clause *cl);
boolean in_key_set(int key_set, struct clause *cl);
void init_a_partition(struct partition *part_ptr);
void init_partitions();
void init_sizes(int maxiset);
void usable_part_insert_all(struct clause *cl);
void sos_part_insert_all(struct clause *cl);
void usable_part_delete_all(struct clause *cl);
void sos_part_delete_all(struct clause *cl);
void scott_delete_from_base(struct clause *cl);
void scott_insert(struct list *lptr, struct clause *cl);
void scott_delete(struct list *lptr, struct clause *cl);
void usable_part_insert(struct clause *cl, int new_maxiset);
void sos_part_insert(struct clause *cl, int new_maxiset);
void usable_part_delete(struct clause *cl, int old_maxiset);
void sos_part_delete(struct clause *cl, int old_maxiset);
void sos_part_mv_to_set(struct clause *cl, int old_maxiset);
void usable_part_mv_to_set(struct clause *cl, int old_maxiset);
void init_partition_box(int set_in_list_cell[MAXISET_ARRAY][MAXISET_ARRAY]);
int get_list_part(char *list_name, struct list *lptr,
		  int set_in_list_cell[MAXISET_ARRAY][MAXISET_ARRAY],
		  int cmp_in_list_cell[MAXISET_ARRAY][MAXISET_ARRAY]);
int compare_partitions(char *list_name,
		       int set_in_list_cell[MAXISET_ARRAY][MAXISET_ARRAY],
		       int set_in_list_cell_copy[MAXISET_ARRAY][MAXISET_ARRAY],
		       int cmp_in_list_cell[MAXISET_ARRAY][MAXISET_ARRAY],
		       int cmp_in_list_cell_copy[MAXISET_ARRAY][MAXISET_ARRAY]);
void debug_partition(char *event);


/*--prototypes in scott_dir/scan.c--*/

void start_scan(int new_maxiset);
void scan_new_base_clauses(int new_maxiset);
void scan_conjecture(int new_maxiset);
void scan_base(int new_maxiset);
void scan_sos(int new_maxiset);
void scan_usable(int new_maxiset);
void scan_first(int new_maxiset);
void invert_existing_base();
void print_scott_sets(FILE *fp, struct clause *cl);
void print_in_set(struct external_list *elist);
void shuffle_ext_list(struct external_list *elist);
void reset_base_markers();
struct clause *search_list(struct list *lptr, 
			   int skip_maxiset,
			   scan_type skip_type);
boolean extend_base(int key_maxiset);
boolean model_clause(struct clause *cl, int maxiset);
boolean test_clause(struct clause *cl, int maxiset);
void check_null_set_size(char *event);


/*--prototypes in scott_dir/scott.c--*/

int semantic_wt(struct clause *cl);
void add_semantic_wt(struct clause *cl);
void retest_failed_clauses(int maxiset);
void remodel_failed_sos_clauses(int maxiset);
void check_failed_clauses();
struct clause *semantic_selection(struct list *lptr);
struct clause *find_first_sem_cl(struct list *lptr, int set_to_skip);
struct clause *find_best_sem_cl(struct list *lptr, int set_to_skip);
int execute_strategy(int_function strategy);
void update_partition(int new_maxiset);
void reset_partition(int bad_maxiset);
boolean keep(int new_maxiset, char *arrow);
int cover_new_clauses(int key);
int refine_choice(int key);
boolean discard_bad_maxisets();
void discard_subsets(int new_maxiset);
void discard(int bad_maxiset);
void disable_semantics(char *event);
void init_semantics(int argc, char *argv[]);
void output_scott_stats(FILE *fp, int stats_level);
void print_special_stats(FILE *fp);
void scott_clock_init();
void scott_clock_reset(int c);
long scott_clock_val(int c);
void message(int priority, char *msg, ...);
void abort_scott(char *msg, ...);
void record_event(char *event, ...);
void output_history();
void print_scott_banner(int argc, char **argv);
int get_num_maxisets();
boolean in_constraint_set(struct clause *cl);


/*--prototypes in scott_dir/sem_res.c--*/

void set_up_model();
boolean false_in_model(struct clause *cl);
void sem_res(struct clause *giv_cl);
void sem_res_recurse(struct clause *nuc, boolean start, struct literal *last_nuc_lit);
struct clause *build_sem_res(struct term *nuc_atom,
			     struct context *nuc_subst,
			     struct term *sat_atom,
			     struct context *sat_subst,
			     boolean start,
			     struct literal *last_nuc_lit,
			     struct literal **new_last_nuc_lit);
void disable_sem_res();


